<?php
/*
  $Id$
  
  Input-Output lab

  Write a php program that does all of the following:
  * can be run as either a command line script (accept args from $argv) or as a GET param’ed Web URL.
  * the script will be passed one directory in the filesystem.
  * if the directory doesn’t exist, you need to output an error immediately to the user & exit.
  * if the directory does exist, then you will output one line per “item” (file or directory) inside of that directory, and you need to differentiate between the two.
  * the script will also accept two additional parameters – one for a “directory match” and one for a “file match”. For every directory, if the directory matches the optional “directory match” parameter, you’ll indicate this to the user.
  * For every file, if the contents of the file match the “file match” parameter (one or more times), you’ll need to indicate this to the user. The ‘strstr’ function and ‘strpos’ functions will both help with finding matches. Is there any benefit to using one over the other?


  @author rICh <rich@quicloud.com>
  */

// set up defaults (best practice)
$lineterminator = "\n";
$dir_match = $file_match = '';
$dir_match_check = $file_match_check = false;

// validate dir & die if not present
if(isset($_GET['dir']) && (!empty($_GET['dir']))) {
  $location = $_GET['dir'];
  $lineterminator = '<br/>';
} else if(isset($argv) && (isset($argv[1])) && (!empty($argv[1]))) {
  $location = $argv[1];
} else {
  // dir is required, die here if not set
  error_message(false,'\n<br>');
}

// validate & set up dir_match
if(isset($_GET['dir_match']) && (!empty($_GET['dir_match']))) {
  $dir_match = $_GET['dir_match'];
  $dir_match_check = true;
} else if(isset($argv) && (isset($argv[2])) && (!empty($argv[2]))) {
  $dir_match = $argv[2];
  $dir_match_check = true;
}

// validate & set up file_match
if(isset($_GET['file_match']) && (!empty($_GET['file_match']))) {
  $file_match = $_GET['file_match'];
  $file_match_check = true;
} else if(isset($argv) && (isset($argv[3])) && (!empty($argv[3]))) {
  $file_match = $argv[3];
  $file_match_check = true;
}

// prints an error message and dies
function error_message($permissions_error = false,$lineterminator,$location = '') {
  if($permissions_error) {
    print "Permissions error encountered: $location doesn't exist or is not readable";
  } else {
    // looks like a useage error
    print "Usage:$lineterminator";
    print "Command line: lab1-input-output.php dir [dir_match] [file_match] $lineterminator";
    print "Web: script?dir=NAME_OF_DIR&dir_match=DIR_MATCH&file_match=FILEMATCH$lineterminator";
    print "In both cases, the dir_match and file_match params are optional";
  }
  exit(); // kill the script here;
}

// Open the directory
$dir_handle = opendir($location);
// if dir doesn't exist or is not readable, opendir will return false
//var_dump($dir_handle); exit();
if(!$dir_handle) {
  error_message(true,$lineterminator,$location);
}

// if we've made it this far, we're really safe & we can now process
// Loop through the directory
while ($entry = readdir($dir_handle)) {
  $matches = ''; // placeholder for our 'MATCHES' param, initialized on every loop
  // If $entry is a directory...
  if (is_dir($location . $entry) && ($entry != ".") && ($entry != "..") ) {
    // strpos is more performant than strstr, but you have to watch out for matches on '0'
    // eg: if dir was 'mydir' and dir_match was 'mydir', then strpos would return "0" (which, withouth the !== false, would be "logically" false because 0 = false). 0 does NOT, however === false
    if($dir_match_check && (strpos($entry,$dir_match) !== false)) {
      $matches = " <b>(DIR NAME MATCHES '$dir_match')</b>";
    }
    print '[Dir] ' . $entry . $matches . $lineterminator;
    // If $entry is a file...
  } else if (is_file($location . $entry)) {
    if($file_match_check) {
      // file matching is a bit tougher, we've got to grab the contents
      //&& 
      $file_contents = file_get_contents($location . $entry);
      if (strpos($file_contents,$file_match) !== false) {
	$matches = " <b>(FILE CONTENTS MATCH '$file_match')</b>";
      }
    }
    
    print '[File] ' . $entry . $matches . $lineterminator;
  }
}
// Close the dir when finished
closedir($dir_handle);

print $lineterminator . 'Done!';
?>